<?php
/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 * 
 * 
 * 
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */

/**
 * backward compatibility: the DIR_SEP constant isn't used anymore
 */
if (! defined('DIR_SEP')) {
	define('DIR_SEP', DIRECTORY_SEPARATOR);
}
/**
 * backward compatibility: the PATH_SEPARATOR constant is availble since 4.3.0RC2
 */
if (! defined('PATH_SEPARATOR')) {
	define('PATH_SEPARATOR', OS_WINDOWS ? ';' : ':');
}

/**
 * Set the KREXX_DIR constant up with the absolute path to kreXX files.
 * If it is
 * not defined, include_path will be used. Set KREXX_DIR only if any other module
 * or application has not already set it up.
 */
if (! defined('KREXX_DIR')) {
	define('KREXX_DIR', dirname(__FILE__) . DIRECTORY_SEPARATOR);
}

/**
 * This constant sets the maximum strings of strings that will be shown
 * as they are.
 * Longer strings will be truncated with this length, and
 * their `full form` will be shown in a child node.
 */
if (! defined('KREXX_TRUNCATE_LENGTH')) {
	define('KREXX_TRUNCATE_LENGTH', 50);
}

/**
 * Register an alias function for object analysis, so you will not have
 * to type \kreXX::open($data); all the time
 *
 *
 * @param mixed $data        	
 */
function krexx($data, $handle = '') {
	if ($handle == '') {
		\kreXX::open($data);
	} else {
		\kreXX::$handle($data);
	}
}

include_once KREXX_DIR . 'kreXXHelp.php';
include_once KREXX_DIR . 'kreXXRender.php';
include_once KREXX_DIR . 'kreXXHive.php';
include_once KREXX_DIR . 'kreXXConfig.php';
include_once KREXX_DIR . 'kreXXToolbox.php';
include_once KREXX_DIR . 'kreXXInternals.php';
include_once KREXX_DIR . 'kreXXObjects.php';
include_once KREXX_DIR . 'kreXXVariables.php';

/**
 * kreXX interface
 *
 * This class hosts the public functions which allow access to the kreXX debug features
 *
 * @package kreXX
 */
use kreSpace\kreXXToolbox;
use kreSpace\kreXXInternals;

class kreXX {
	
	/**
	 * Here we save all timekeeping stuff
	 *
	 * @var string array
	 */
	protected static $timekeeping = array ();
	
	/**
	 * Remembers, if the header or footer must be printed
	 *
	 * @var integer
	 */
	protected static $startRun = 0;
	protected static $counterCache = array ();
	
	/**
	 * Handles the developer handle
	 *
	 * @param string $name        	
	 * @param array $arguments        	
	 */
	public static function __callStatic($name, $arguments) {
		// do we gave a handle?
		$handle = kreSpace\kreXXConfig::getConfigFromCookies('deep', 'Local open function');
		if ($name == $handle) {
			// we do a standard-open
			if (isset($arguments[0])) {
				self::open($arguments[0]);
			} else {
				self::open();
			}
		} else {
			// do nothing
		}
	}
	
	/**
	 * Resets the timer and takes a "moment"
	 * 
	 * @param string $string        	
	 */
	public Static function timerStart() {
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// reset what we had before.
		self::$timekeeping = array ();
		self::$counterCache = array ();
		self::$timekeeping[] = $_SERVER['REQUEST_URI'];
		
		self::timerMoment('start');
	}
	
	/**
	 * Takes a "moment"
	 * 
	 * @param string $string        	
	 */
	public static function timerMoment($string) {
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// static $counterCache = array();
		
		// did we use this one before?
		if (isset(self::$counterCache[$string])) {
			// add another to the counter
			self::$counterCache[$string]++;
			self::$timekeeping['[' . self::$counterCache[$string] . ']' . $string] = microtime(true);
		} else {
			// first time counter, set it to 1
			self::$counterCache[$string] = 1;
			self::$timekeeping[$string] = microtime(true);
		}
	}
	
	/**
	 * Takes a "moment" and outputs the timer in a analyse file
	 * 
	 * @param unknown $string        	
	 */
	public static function timerEnd() {
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		self::timerMoment('end');
		// . . . and we are done
		kreXXInternals::dump(kreXXInternals::miniBenchTo(self::$timekeeping), 'kreXX timer');
	}
	
	/**
	 * Dumps the SQL Log
	 *
	 * @param \string $uniqueId
	 *        	ID of the Log, leave blank for everything (which micht be a little too much, uncached ~350 entries)
	 * @param \integer $count
	 *        	Anzahl der auzugerbenen Queries. Bei Drupal können es mal locker über 600 Stück sein, was eher sinnlos ist, und eventuell den PHP Speicher sprengt.
	 *        	
	 */
	public static function dumpSql($uniqueId = 'krexx', $count = 0) {
		static $oldData = array ();
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render Typo3, if possible
		//
		// if (isset($GLOBALS['TYPO3_DB']->debug_lastBuiltQuery)) {
		// kreXX::dump($GLOBALS['TYPO3_DB']->debug_lastBuiltQuery, 'Last SQL Query from Typo3');
		// }
		
		// render Drupal, if possible
		//
		if (class_exists('Database') && method_exists('Database', 'getLog')) {
			$data = Database::getLog($uniqueId);
			if (! is_array($data)) {
				// huh, no logs available
				$data = array ();
			}
			$oldData = array_merge($oldData, $data);
			if ((int) $count == 0) {
				// Voreinstellungen aus dem Framework nehmen
				$countQueries = variable_get('includekrexx_count_queries', 0);
				$data = array_slice($oldData, $countQueries * - 1, $countQueries, true);
			}
			Database::startLog('krexx');
			
			kreXXInternals::dump($data, 'Last SQL Query from Drupal');
		}
	}
	
	/**
	 * Prints all methods and magic functions in this class
	 * Wrapper for the Dump method
	 *
	 * @param mixed $data,...        	
	 * @access public
	 */
	Public Static function open($data = null) {
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		kreXXInternals::dump($data);
	}
	
	/**
	 * Starts the Drupal SQL Logging.
	 * The $uniqueId can be used to narrow down the dumpSql function
	 *
	 * @param \string $uniqueId        	
	 */
	public static function startDrupalLogging($uniqueId = 'krexx') {
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		if (class_exists('Database') && method_exists('Database', 'startLog')) {
			include_once DRUPAL_ROOT . '/includes/database/log.inc';
			Database::startLog($uniqueId);
		} else {
			// sure this is a Drupal?
			kreXXInternals::dump('Are you sure that this is a Drupal?');
		}
	}
	
	/**
	 * Prints out all currently defined global variables
	 */
	public static function variables() {
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump($GLOBALS, 'Global variables');
	}
	
	/**
	 * Prints a debug backtrace
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function backtrace() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(debug_backtrace(), 'Backtrace');
	}
	
	/**
	 * Prints a list of all currently declared classes.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function classes() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(get_declared_classes(), 'Declared classes');
	}
	
	/**
	 * Prints a list of all currently declared interfaces (PHP5 only).
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function interfaces() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(get_declared_interfaces(), 'Declared unterfaces');
	}
	
	/**
	 * Prints a list of all currently included (or required) files.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function includes() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(get_included_files(), 'Included (or required) files');
	}
	
	/**
	 * Prints a list of all currently declared functions.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function functions() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(get_defined_functions(), 'Declared functions');
	}
	
	/**
	 * Prints a list of all currently declared constants.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function defines() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(get_defined_constants(), 'Declared constants (defines)');
	}
	
	/**
	 * Prints a list of all currently loaded PHP extensions.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function extensions() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}

		// render it
		//
		kreXXInternals::dump(get_loaded_extensions(), 'Loaded PHP extensions');
	}
	
	/**
	 * Prints a list of all HTTP request headers.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function headers() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(getallheaders(), 'HTTP request headers');
	}
	
	/**
	 * Prints a list of the configuration settings read from <i>php.ini</i>
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function phpini() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		$what = 'Configuration settings read from ' . get_cfg_var('cfg_file_path');
		if (get_cfg_var('cfg_file_path') === false) {
			kreXXInternals::dump('Phpini is not readable.', $what);
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(parse_ini_file(get_cfg_var('cfg_file_path'), true), $what);
	}
	
	/**
	 * Prints a list of all your configuration settings.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function conf() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		kreXXInternals::dump(ini_get_all(), 'Configuration settings');
	}
	
	/**
	 * Prints a list of the specified directories under your <i>include_path</i> option.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function path() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Specified directories under your include_path option.';
		kreXXInternals::dump(explode(PATH_SEPARATOR, ini_get('include_path')), $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_REQUEST</i> array.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function request() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the $_REQUEST array.';
		kreXXInternals::dump($_REQUEST, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_GET</i> array.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function get() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		// render it
		//
		$what = 'Values from the $_GET array';
		kreXXInternals::dump($_GET, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_POST</i> array.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function post() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the $_POST array';
		kreXXInternals::dump($_POST, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_SERVER</i> array.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function server() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the $_SERVER array';
		kreXXInternals::dump($_SERVER, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_COOKIE</i> array.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function cookie() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the $_COOKIE array';
		kreXXInternals::dump($_COOKIE, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_ENV</i> array.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function env() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the $_ENV array';
		kreXXInternals::dump($_ENV, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_SESSION</i> array.
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function session() {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the $_SESSION array';
		kreXXInternals::dump($_SESSION, $what);
	}
	
	/**
	 * Prints a list of all the values from an INI file.
	 *
	 * @param string $ini_file        	
	 *
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function ini($ini_file) {
		
		// disabled ?
		//
		if (! kreXXToolbox::isEnabled()) {
			return;
		}
		
		// read it
		//
		if (! $_ = @parse_ini_file($ini_file, 1)) {
			kreXXInternals::dump('File "' . $ini_file . '" is not readable.', $what);
			return;
		}
		
		// render it
		//
		$what = 'Values from the ' . realpath($ini_file) ? realpath($ini_file) : $ini_file . ' INI file.';
		kreXXInternals::dump($_, $what);
	}
	
	/**
	 * Enable kreXX
	 *
	 * @return boolean
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function enable() {
		return true === kreXXToolbox::isEnabled(true);
	}
	
	/**
	 * Disable kreXX
	 *
	 * @return boolean
	 * @access public
	 * @static
	 *
	 *
	 */
	Public Static Function disable() {
		return false === kreXXToolbox::isEnabled(false);
	}
}
