<?php
/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 *
 *
 *
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */
namespace kreSpace;

/**
 * kreXX hive
 *
 * This class hosts the kreXX configuration
 *
 * @package kreXX
 */
class kreXXConfig {
	
	/**
	 * Fallback settings, in case there is nothing in the config ini
	 *
	 * @var array
	 */
	protected static $configFallback = array (
			'render' => array (
					'skin' => 'schablon.com',
					'doctype' => '<!DOCTYPE html>',
					'jsLib' => 'jquery-1.11.0.js'
			),
			'logging' => array (
					'folder' => 'log',
					'maxfiles' => '10'
			),
			'output' => array (
					'destination' => 'frontend',
					'useCookies' => 'false',
					'maxCall' => '10',
					'disabled' => 'false',
			),
			'deep' => array (
					'analyseProtected' => 'false',
					'analysePrivate' => 'false',
					'analyseMethods' => 'true',
					'analyseTraversable' => 'true',
					'debugMethods' => 'debug,toArray,__toString,toString',
					'level' => '5'
			)
	);
	
	/**
	 * Path to the configuration file
	 * 
	 * @var string
	 */
	protected static $pathToIni;
	
	
	/**
	 * Returns values from kreXX's configuration
	 *
	 * @param string $group
	 * @param string $name
	 * @return string
	 *
	 * @access protected
	 * @static
	 *
	 */
	public Static Function getConfigValue($group, $name) {
		static $config = array();
		// do some caching
		if (isset($config[$group][$name])){
			return $config[$group][$name];
		}
	
		// do we have a value in the cookies?
		$localSetting = self::getConfigFromCookies($group, $name);
		if (isset($localSetting)) {
			$config[$group][$name] = $localSetting;
			return $localSetting;
		}
	
		// do we have a value in the ini?
		$iniSettings = self::getConfigFromFile($group, $name);
		if (isset($iniSettings)) {
			$config[$group][$name] = $iniSettings;
			return $iniSettings;
		}
	
		// nothing yet? give back factory settings
		$config[$group][$name] = self::$configFallback[$group][$name];
		return self::$configFallback[$group][$name];
	}
	
	/**
	 * 
	 * @return array the configuration with the source
	 */
	public static function getWholeConfiguration() {
		// we may have some project settings in the ini
		// as well as some in the cookies,
		// but some may be missing (factory), so we need to parse
		// these settings in there
		$source = array();
		$config = array();
		
		foreach (self::$configFallback as $sectionName => $sectionData) {
			foreach ($sectionData as $parameterName => $parameterValue) {
				// get cookie settings
				$cookieValue = self::getConfigFromCookies($sectionName, $parameterName);
				if (isset($cookieValue)) {
					$config[$sectionName][$parameterName] = $cookieValue;
					$source[$sectionName][$parameterName] = 'local cookie settings';
					continue;
				}
				$iniValue = self::getConfigFromFile($sectionName, $parameterName);
				// get ini settings
				if (isset($iniValue)) {
					$config[$sectionName][$parameterName] = $iniValue;
					$source[$sectionName][$parameterName] = 'kreXX ini settings';
					continue;
				}
				// nothing yet? return factory settings.
				$config[$sectionName][$parameterName] = $parameterValue;
				$source[$sectionName][$parameterName] = 'factory settings';
		
			}
		}
		$result = array($source, $config);
		return $result;
	}
	
	public static function getDevHandler() {
		return self::getConfigFromCookies('deep', 'Local open function');
	}
	
	/**
	 * Gets the path to the inifile
	 * 
	 * @return string
	 */
	public Static Function getPathToIni() {
		return self::$pathToIni;
	}
	
	/**
	 * Returns settings from the ini file
	 *
	 * @param string $group
	 * @param string $name
	 * @return void|string
	 */
	protected static function getConfigFromFile($group, $name) {
		static $_config = array ();
	
		// not loaded ?
		if (empty($_config)) {
			// file is somewhere else
			$configIni = (array) @parse_ini_file(KREXX_DIR . 'kreXXconfig.ini', true);
			if (isset($configIni['pathtoini']['pathtoini'])) {
				self::$pathToIni = $configIni['pathtoini']['pathtoini'];
			} else {
				self::$pathToIni = KREXX_DIR . 'kreXX.ini';
			}
			$_config = (array) @parse_ini_file(self::$pathToIni, true);
		}
	
		// do we have a value in the ini?
		if (isset($_config[$group][$name])) {
			return $_config[$group][$name];
		}
	
	}
	
	/**
	 * Returns settings from the local cookies
	 *
	 * @param string $group
	 * @param string $name
	 * @return void|string
	 */
	public static function getConfigFromCookies($group, $name) {
		static $_config = array ();
	
		if (!isset($_COOKIE['kreXXDebugSettings'])) {
			// no cookies, no result
			return;
		}
	
	
		// not loaded ?
		if (empty($_config)) {
			// we have local settings
			$_config = json_decode($_COOKIE['kreXXDebugSettings'], true);
		}
	
		// do we have a value in the cookies?
		// we only accept settings from the 'deep' part, to prevent manipulation
		if ($group == 'deep') {
			// we escape them, just in case
			if (isset($_config[$name])) {
				return htmlspecialchars($_config[$name]);
			}
		}
	}
}