<?php
/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 *
 *
 *
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */
namespace kreSpace;

/**
 * kreXX hive
 *
 * This class decides about recussions
 *
 * @package kreXX
 */
class kreXXHive {
	/**
	 * Here we save all arrays, as well as all object
	 * hashes, to prevent recussions
	 * [0] -> array with markers
	 * [1] -> object hashes
	 *
	 * @var array
	 */
	protected static $recussionHive = array ();
	
	/**
	 * Adds a variable to the hive of arrays and objects which
	 * are tracked for whether they have recursive entries
	 *
	 * @param mixed &$bee either array or object, not a scallar vale
	 * @return array all the bees
	 */
	public Static Function &addToHive(&$bee) {
	
		// static $_ = array ();
	
		// new bee ?
		//
		if (is_array($bee) || is_object($bee)) {
	
			// stain it
			$_recursion_marker = self::getMarker();
			if (is_object($bee)) {
				// we do something else for objects
				// setting a recussion er inside might trigger a magical function.
				// some Zend Framework objects throw an error, while Varien Objects
				// redirect the recussion value to an internal array
				$objectHash = spl_object_hash($bee);
				if (! isset(self::$recussionHive[1][$objectHash])) {
					self::$recussionHive[1][$objectHash] = 0;
				}
				self::$recussionHive[1][$objectHash]++;
			} else {
				// this should be an array
				if (! isset($bee[$_recursion_marker])) {
					$bee[$_recursion_marker] = 0;
				}
				$bee[$_recursion_marker]++;
				self::$recussionHive[0][] = & $bee;
			}
		}
		return $bee;
	}
	
	/**
	 * Resets all Arrays inside the recussion array
	 */
	public static function cleanupHive() {
		// remove all recussion marker inside of arrays
		$recursionMarker = self::getMarker();
		if (isset(self::$recussionHive[0]) && count(self::$recussionHive[0])) {
			foreach (self::$recussionHive[0] as $i => $bee) {
				try {
					unset(self::$recussionHive[0][$i][$recursionMarker]);
				} catch (Exception $e) {
					// do nothing
				}
			}
		}
		// reset the array, to get it clean for the next run
		self::$recussionHive = array ();
	}
	
	/**
	 * Find out if our bee is already in the hive
	 * Do we have a recussion here?
	 *
	 * @param mixed $bee
	 * @return boolean
	 */
	public static function isInHive($bee) {
		// test for references in order to
		// prevent endless recursion loops
		//
		$recursionMarker = self::getMarker();
		if (is_object($bee)) {
			// retrieve a possible hash
			$objectHash = spl_object_hash($bee);
			$recussionValue = @self::$recussionHive[1][$objectHash];
		} else {
			// retrieve a possible value
			if (isset($bee[$recursionMarker])) {
				$recussionValue = $bee[$recursionMarker];
			} else {
				$recussionValue = 0;
			}
		}
	
		$recussionValue = (int) $recussionValue;
		if ($recussionValue > 0) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Return the marked used to stain arrays
	 * and objects in order to detect recursions
	 *
	 * @return string
	 */
	public Static Function getMarker() {
		static $_recursion_marker;
		if (! isset($_recursion_marker)) {
			$_recursion_marker = uniqid('kreXX');
		}
	
		return $_recursion_marker;
	}
}