<?php

/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 *
 *
 *
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */
namespace kreSpace;

/**
 * kreXX internals
 *
 * This class hosts the internal analysis functions
 *
 * @package kreXX
 */
class kreXXInternals {
	
	/**
	 * Unix timestamp, used to determine if we need to do an
	 * emergency break
	 *
	 * @var integer
	 */
	protected static $timer;

	/**
	 *
	 * @param unknown $arg_t        	
	 * @param string $arg_ra        	
	 * @return array
	 * @see http://php.net/manual/de/function.microtime.php
	 * @author gomodo at free dot fr
	 */
	public static function miniBenchTo($arg_t) {
		$tttime = round((end($arg_t) - $arg_t['start']) * 1000, 4);
		$ar_aff['url'] = $_SERVER['REQUEST_URI'];
		$ar_aff['total_time'] = $tttime;
		$prv_cle = 'start';
		$prv_val = $arg_t['start'];
		
		foreach ($arg_t as $cle => $val) {
			if ($cle != 'start') {
				// calculate thie time
				$prcnt_t = round(((round(($val - $prv_val) * 1000, 4) / $tttime) * 100), 1);
				$ar_aff[$prv_cle . ' -> ' . $cle] = $prcnt_t . '%';
				$prv_val = $val;
				$prv_cle = $cle;
			}
		}
		return $ar_aff;
	}
	


	
	/**
	 * Dump information about a variable
	 * This function  decides what functions analyse the data
	 * and acts as a hub.
	 *
	 * @param mixed $data        	
	 * @param string $name
	 * @return string the generated DOM
	 *
	 */
	public Static Function analysisHub(&$data, $name = '...') {
		// check emergency break, 60 seconds should be enough
		if (self::$timer + 60 < time()) {
			// no other calls in this run!
			kreXXToolbox::isEnabled(false);
			// tell the user what went wrong, this should close the DOM nicely
			return kreXXVariables::analyseString('Emergency break for large output', $name);	
		}
		// check memory usage
		if (!self::isEnoughMemoryLeft()) {
			// tell the user what went wrong, this should close the DOM nicely
			return kreXXVariables::analyseString('Emergency break for large memory usage', $name);
		}
		
		// object ?
		//
		if (is_object($data)) {
			return kreXXObjects::analyseObject($data, $name);
		}
		
		// array ?
		//
		if (is_array($data)) {
			return kreXXVariables::analyseArray($data, $name);
		}
		
		// resource ?
		//
		if (is_resource($data)) {
			return kreXXVariables::analyseResource($data, $name);
		}
		
		// scalar ?
		//
		if (is_string($data)) {
			return kreXXVariables::analyseString($data, $name);
		}
		
		if (is_float($data)) {
			return kreXXVariables::analyseFloat($data, $name);
		}
		
		if (is_integer($data)) {
			return kreXXVariables::analyseInteger($data, $name);
		}
		
		if (is_bool($data)) {
			return kreXXVariables::analyseBoolean($data, $name);
		}
		
		// null ?
		//
		if (is_null($data)) {
			return kreXXVariables::analyseNull($name);
		}
	}
	

	
	/**
	 * Render a dump for the properties of an array or object
	 *
	 * @param mixed &$data
	 */
	public Static Function interateThrough(&$data) {
		$parameter = array (
				$data 
		);
		$analysis = function (&$parameter) {
			$output = '';
			$data = $parameter[0];
			$isObject = is_object($data);
			
			$recursionMarker = kreXXHive::getMarker();
			
			// recursion detection of objects are
			// handeld in _object()
			if (is_array($data) && kreXXHive::isInHive($data)) {
				return kreXXRender::renderRecursion();
			}
			
			// remember, that we've already been here
			kreXXHive::addToHive($data);
			
			// keys ?
			$keys = ($isObject) ? array_keys(get_object_vars($data)) : array_keys($data);
			
			// itterate
			foreach ($keys as $k) {
				
				// skip marker
				if ($k === $recursionMarker) {
					continue;
				}
				
				// get real value
				if ($isObject) {
					$v = & $data->$k;
				} else {
					$v = & $data[$k];
				}
				
				$output .= kreXXInternals::analysisHub($v, $k);
			}
			return $output;
		};
		return kreXXRender::renderExpandableChild('', '', $analysis, $parameter);
	}
	
	
	/**
	 * Dump information about a variable
	 *
	 * @param mixed $data
	 * @param string $headline
	 */
	public Static Function dump($data, $headline = '') {
		
		// start timer
		// when a certain time has passed, kreXX will use an
		// emergency break to prevent too large output (or no output at all (WSOD)
		self::$timer = time();
		
		// find caller
		$_ = debug_backtrace();
		while ($caller = array_pop($_)) {
			if (strToLower(@$caller['function']) == 'krexx' || strToLower(@$caller['class']) == 'krexx') {
				break;
			}
		}
		
		// set the headline, if it's not set already
		if ($headline == '') {
			if (is_object($data)) {
				$headline = get_class($data);
			}
			if (is_array($data)) {
				$headline = 'array';
			}
			if (is_bool($data)) {
				$headline = 'boolean';
			}
			if (is_float($data)) {
				$headline = 'float';
			}
			if (is_int($data)) {
				$headline = 'integer';
			}
			if (is_null($data)) {
				$headline = 'null';
			}
			if (is_resource($data)) {
				$headline = 'resource';
			}
			if (is_string($data)) {
				$headline = 'string';
			}
		}
		
		$maxCall = (int) kreXXConfig::getConfigValue('output', 'maxCall');
		if (kreXXRender::$kreXXCount >= $maxCall) {
			// called too often, we might get into trouble here!
			return;
		}
		
		// Start Output
		kreXXRender::$kreXXCount++;
		$output = kreXXToolbox::outputHeader($headline);
		$output .= kreXXToolbox::getCssAndJs();
		$output .= self::analysisHub($data);
		$output .= kreXXToolbox::outputFooter($caller);
		if (kreXXConfig::getConfigValue('output', 'destination') == 'file') {
			// save it to a file
			kreXXToolbox::saveOutputToFile($output);
		} else {
			// send it to the browser
			echo $output;
		}
		
		// typo3 6.2 renders only as much characters as it has
		// inside the content cache. kreXX has no access there,
		// so anything additional will delete stuff from the page button.
		// We flush it only once, to prevent a notice
		if (kreXXRender::$kreXXCount == 1) {
			ob_end_flush();
		}
		
		// cleanup
		kreXXHive::cleanupHive();
	}
	
	/**
	 * 
	 * @return boolean
	 */
	protected function isEnoughMemoryLeft() {
		$limit = ini_get('memory_limit');
		if($limit === false) {
			// no vaule?
			return true;
		}
		if (preg_match('/^(\d+)(.)$/', $limit, $matches)) {
	    	if ($matches[2] == 'M') {
	    		// Megabyte
	        	$memoryLimit = $matches[1] * 1024 * 1024;
	    	} else if ($matches[2] == 'K') {
	    		// Kilobyte
	        	$memoryLimit = $matches[1] * 1024;
	    	}
		}
		$usage = memory_get_usage();
		$left = $memoryLimit - $usage;
		// is at least 16MB left?
		return $left >= 16 * 1024 * 1024;
	}
	
}