<?php

/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 *
 *
 *
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */
namespace kreSpace;

/**
 * kreXX objects
 *
 * This class hosts the object analysis functions
 *
 * @package kreXX
 */
class kreXXObjects {
	
	/**
	 * Render a dump for an object
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 *
	 * @todo refactor this method, it's too big
	 */
	public Static Function analyseObject($data, $name) {
		static $level = 0;
		
		$output = '';
		$parameter = array (
				$data,
				$name 
		);
		$level++;
		
		if (kreXXHive::isInHive($data)) {
			// tell them, we've been here before
			// but also say who we are
			$output .= kreXXRender::renderRecursion($name, get_class($data), kreXXToolbox::generateDomIdFromObject($data));
			
			// we will not render this one, but since we
			// return to wherever we came from, we need to decrese the level
			$level--;
			return $output;
		}
		// remember, that we've been here before
		kreXXHive::addToHive($data);
		
		// analyse it only, if we are below a certain level.
		if ($level <= (int) kreXXConfig::getConfigValue('deep', 'level')) {
			$anonFunction = function (&$parameter) {
				$data = $parameter[0];
				$name = $parameter[1];
				$output = '';
				
				// Dumping all Properties
				// But only if we have any
				if (count(get_object_vars($data))) {
					$parameter = array (
							$data 
					);
					$anonFunction = function (&$parameter) {
						$data = $parameter[0];
						// standard dump of the vars
						return kreXXInternals::interateThrough($data);
					};
					$output .= kreXXRender::renderExpandableChild($name, 'class internals', $anonFunction, $parameter, 'Public properties');
				}
				
				// Dumping all protected properties
				if (kreXXConfig::getConfigValue('deep', 'analyseProtected') == 'true') {
					$ref = new \ReflectionClass($data);
					$refProps = $ref->getProperties(\ReflectionProperty::IS_PROTECTED);
					if (count($refProps)) {
						$output .= kreXXObjects::getReflectionPropertiesData($refProps, $name, $ref, $data, 'Protected properties');
					}
				}
				
				// Dumping all private properties
				if (kreXXConfig::getConfigValue('deep', 'analysePrivate') == 'true') {
					$ref = new \ReflectionClass($data);
					$refProps = $ref->getProperties(\ReflectionProperty::IS_PRIVATE);
					if (count($refProps)) {
						$output .= kreXXObjects::getReflectionPropertiesData($refProps, $name, $ref, $data, 'Private properties');
					}
				}
				
				// Dumping all methods
				// But only if we have any
				$output .= kreXXObjects::getMethodData($data, $name);
				
				// Dumping traversable data
				if (kreXXConfig::getConfigValue('deep', 'analyseTraversable') == 'true') {
					$output .= kreXXObjects::getTraversableData($data, $name);
				}
				
				// Dumping all configured debug functions
				$output .= kreXXObjects::pollAllConfiguredDebugMethods($data, $name);
				return $output;
			};
		} else {
			// we will not go into the object pyramide any further
			$anonFunction = function () {
				return kreXXVariables::analyseString("Maxlevel for object analysis reached. I will not go any further.\n To increase this value, please edit the ini file.", 'Feedback');
			};
		}
		// output data from the class
		$output .= kreXXRender::renderExpandableChild($name, 'class', $anonFunction, $parameter, get_class($data), kreXXToolbox::generateDomIdFromObject($data));
		// we've finished this one, and can decrease the levelsetting
		$level--;
		return $output;
	}
	
	/**
	 * 
	 * @param array $refProps
	 * @param string $name
	 * @param string $label
	 * @return string
	 */
	public static function getReflectionPropertiesData($refProps, $name, $ref, $data, $label) {
		// I need to preprocess them, since I do not want to render a reflection property
		$default = $ref->getDefaultProperties();
		$privateProps = array ();
		foreach ($refProps as $refProerty) {
			$refProerty->setAccessible(true);
			$value = $refProerty->getValue($data);
			$propName = $refProerty->name;
			if (is_null($value)) {
				// no Value is set?
				// we might want to look at the default value
				$value = $default[$propName];
			}
			$privateProps[$propName] = $value;
		}
		
		$parameter = array (
				$privateProps
		);
		$anonFunction = function (&$parameter) {
			$data = $parameter[0];
			// standard dump of the vars
			return kreXXInternals::interateThrough($data);
		};
		return kreXXRender::renderExpandableChild($name, 'class internals', $anonFunction, $parameter, $label);
	}
	
	/**
	 * Dmps all infos about the public methods of an object
	 * 
	 * @param object $data
	 * @param string $name
	 * @return string
	 */
	public static function getMethodData($data, $name) {
		// Dumping all methods
		// But only if we have any
		$methods = get_class_methods($data);
		sort($methods);
		$myMethods = array ();
		foreach ($methods as $k => $v) {
			$myMethods[] = $v;
		}
		
		// Dump them only, if there actually are methods in there
		if (count($myMethods)) {
			if (kreXXConfig::getConfigValue('deep', 'analyseMethods') == 'true') {
				$ref = new \ReflectionClass($data);
			} else {
				// will not be uses, dummy object
				$ref = new \stdClass();
			}
			$parameter = array (
					$myMethods,
					$ref
			);
			$anonFunction = function (&$parameter) {
				$myMethods = $parameter[0];
				$ref = $parameter[1];
				return kreXXObjects::analyseMethods($myMethods, $ref);
			};
			return kreXXRender::renderExpandableChild($name, 'class internals', $anonFunction, $parameter, 'Methods');
		}
	}
	
	
	/**
	 * Dumps all available traversable data
	 * 
	 * @param object $data
	 * @return string
	 */
	public static function getTraversableData($data, $name) {
		if (is_a($data, 'Traversable')) {
			$parameter = iterator_to_array($data);
			$anonFunction = function (&$data) {
				// this could be anything, we need to examine it first, which happens in the _dump()
				return kreXXInternals::analysisHub($data);
			};
			return kreXXRender::renderExpandableChild($name, 'Foreach', $anonFunction, $parameter, 'Traversable Info');
		}
	}
	
	
	/**
	 * Polls all configured debug methods in die class
	 * 
	 * @param object $data
	 * @param string $name
	 * @return string
	 */
	public static function pollAllConfiguredDebugMethods($data, $name) {
		$output = '';
		
		$funcList = explode(',', kreXXConfig::getConfigValue('deep', 'debugMethods'));
		foreach ($funcList as $funcName) {
			if (method_exists($data, $funcName)) {
				// @todo use an errorhandler to catch possible fatals here
				// also, for some reasons does the @ not work in a Magento,
				// strangely, when I poll the magento-debug functions with
				// the default values, I get NULL in return
				// looks like we have to stick with the warning, until I find
				// a method to suppress it
				try {
					$args = array ();
					$parameter = call_user_func_array(array (
							$data,
							$funcName
					), $args);
				} catch (Exception $e) {
					$parameter = 'Could not retrive data from: ' . $funcName;
				}
				$anonFunction = function (&$parameter) {
					return kreXXInternals::analysisHub($parameter);
				};
				$output .= kreXXRender::renderExpandableChild($name, 'debug method', $anonFunction, $parameter, $funcName . ' Info');
			}
		}
		return $output;
	}
	
	/**
	 * Render a dump for method infos
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 */
	public Static Function analyseMethod($data, $name) {
		$parameter = array (
				$data 
		);
		$anonFunction = function ($parameter) {
			$data = $parameter[0];
			$output = '';
			foreach ($data as $key => $string) {
				if ($key !== 'comments' && $key !== 'declared in') {
					$output .= kreXXRender::renderSingleChild($string, $key, $string, false, 'reflection');
				} else {
					// security, there could be anything inside the string
					$output .= kreXXRender::renderSingleChild($string, $key, '. . .', true, 'reflection');
				}
			}
			return $output;
		};
		return kreXXRender::renderExpandableChild($name, 'method', $anonFunction, $parameter);
	}
	
	/**
	 * Render a dump for the methods of an object
	 *
	 * @param array &$data
	 * 		  the methods
	 * @param reflection_class $ref
	 *        a reflection of the original class
	 */
	public Static Function analyseMethods(&$data, $ref) {
		$parameter = array (
				$data,
				$ref 
		);
		$analysis = function (&$parameter) {
			$data = $parameter[0];
			$ref = $parameter[1];
			$result = '';
			// itterate
			//
			if (kreXXConfig::getConfigValue('deep', 'analyseMethods') == 'true') {
				// Deep analysis of the methods
				foreach ($data as $method) {
					$reflection = $ref->getMethod($method);
					$methodData = array ();
					// get the comment from the class or it's parents
					$methodData['comments'] = kreXXObjects::getParentalComment($reflection->getDocComment(), $ref, $method);
					// get the comment from the interfaces, if necessary
					// and clean it up
					$methodData['comments'] = nl2br(htmlSpecialChars(kreXXObjects::getInterfaceComment($methodData['comments'], $ref, $method)));
					// get declaration place
					$declaringClass = $reflection->getDeclaringClass();
					$methodData['declared in'] = htmlSpecialChars($declaringClass->getFileName()) . '<br>';
					$methodData['declared in'] .= htmlSpecialChars($declaringClass->getName()) . ' ';
					$methodData['declared in'] .= 'in line ' . htmlSpecialChars($reflection->getStartLine());
					// get parameters
					$parameters = $reflection->getParameters();
					foreach ($parameters as $parameter) {
						preg_match('/(.*)(?= \[ )/', $parameter, $key);
						$parameter = str_replace($key[0], '', $parameter);
						$methodData[$key[0]] = htmlSpecialChars(trim($parameter, ' []'));
					}
					
					$result .= kreXXObjects::analyseMethod($methodData, $method);
				}
				return $result;
			} else {
				// We just dump them
				return kreXXVariables::analyseArray($data, 'methods');
			}
		};
		return kreXXRender::render($analysis, $parameter);
	}
	
	/**
	 * Gets comments from the reflection
	 * Inherited comments are resolved by recussion of this function
	 *
	 * @param string $originalComment        	
	 * @param reflectionclass $reflection        	
	 * @param string $methodName        	
	 */
	public function getParentalComment($originalComment, $reflection, $methodName) {
		if (stripos($originalComment, '{@inheritdoc}') !== false) {
			// now we need to get the parentclass and the comment from the parent function
			/* @var reflectionclass $parentClass */
			$parentClass = $reflection->getParentClass();
			if (! is_object($parentClass)) {
				// we've gone too far
				// maybe an interface?
				return $originalComment;
			}
			
			try {
				$parentMethod = $parentClass->getMethod($methodName);
				$parentcomment = $parentMethod->getDocComment();
			} catch (\ReflectionException $e) {
				// looks like we are trying to inherit from a not existing method
				// maybe an interface?
				return $originalComment;
			}
			// replace it
			$originalComment = str_ireplace('{@inheritdoc}', $parentcomment, $originalComment);
			// die(get_class($originalComment));
			// and seach for further parental comments . . .
			return kreXXObjects::getParentalComment($originalComment, $parentClass, $methodName);
		\Drupal\Core\Entity\Entity;
		} else {
			// we don't need to do anything with it
			return $originalComment;
		}
	}
	
	/**
	 * Iterated through an array of interfaces, to see, if we can resolve the inherited comment
	 *
	 * @param string $originalComment        	
	 * @param array $interfaceArray
	 *        	array of interfaces
	 * @param string $methodName        	
	 */
	public function getInterfaceComment($originalComment, $reflection, $methodName) {
		$interfaceArray = $reflection->getInterfaces();
		foreach ($interfaceArray as $interface) {
			if (stripos($originalComment, '{@inheritdoc}') !== false) {
				try {
					$interfaceMethod = $interface->getMethod($methodName);
					if (! is_object($interfaceMethod)) {
						// we've gone too far
						// we should tell the user, that we could not resolve the inherited comment
						$originalComment = str_ireplace('{@inheritdoc}', ' ***could not resolve inherited comment*** ', $originalComment);
					} else {
						$interfacecomment = $interfaceMethod->getDocComment();
						// replace it
						$originalComment = str_ireplace('{@inheritdoc}', $interfacecomment, $originalComment);
					}
				} catch (\ReflectionException $e) {
					// Method not found
					// we should try the next interface
				}
			} else {
				// Looks like we've resolved them all
				return $originalComment;
			}
		}
		// when we are still here, we should tell the user, that we could not
		// resolve the inherited comment
		$originalComment = str_ireplace('{@inheritdoc}', ' ***could not resolve inherited comment*** ', $originalComment);
		return $originalComment;
	}
}