<?php

/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 *
 *
 *
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */
namespace kreSpace;


/**
 * kreXX toolbox
 *
 * This class hosts functions, which offer additional services
 *
 * @package kreXX
 */
class kreXXToolbox {
	
	/**
	 * Stores if kreXX is actually enabled
	 *
	 * @var boolean
	 */
	protected static $isEnabled = true;
	
	/**
	 * Get\Set kreXX state: whether it is enabled or disabled
	 *
	 * @param boolean $state
	 * @return boolean
	 * @access protected
	 * @static
	 */
	public Static Function isEnabled($state = null) {
	
		// set
		if (isset($state)) {
			self::$isEnabled = $state;
				
			return self::$isEnabled;
		}
	
		// disabled globally in the ini?
		if (kreXXConfig::getConfigValue('output', 'disabled') == 'true') {
			self::$isEnabled = false;
			return false;
		}
	
		// cookies?
		if (kreXXConfig::getConfigValue('output', 'useCookies') == 'true') {
			if (isset($_COOKIE['kreXXDebug']) && $_COOKIE['kreXXDebug'] == 'yes'){		// Debug cookie there?
				$debugSession = true;
			} else {
				$debugSession = false;
			}
	
			if ($debugSession === true && self::$isEnabled === true) {
				return true;
			} else {
				return false;
			}
		}
		// not using cookies
		return self::$isEnabled;
	}
	
	/**
	 * Logges the output of kreXX to a file and cleans up and
	 * cleans up the output folder
	 *
	 * @param string $output
	 */
	public static function saveOutputToFile($output) {
		static $logDir;
		if (is_null($logDir)) {
			$logDir = kreXXConfig::getConfigValue('logging', 'folder') . DIRECTORY_SEPARATOR;
		}
	
		$logList = glob(KREXX_DIR . $logDir . "*");
		array_multisort(array_map('filemtime', $logList), SORT_DESC, $logList);
		$maxFileCount = (int) kreXXConfig::getConfigValue('logging', 'maxfiles');
		$count = 1;
		foreach ($logList as $file) {
			if (is_file($file) && $count >= $maxFileCount) {
				unlink($file);
			}
			$count++;
		}
	
		static $timestamp = 0;
	
		if ($timestamp == 0) {
			$timestamp = time();
		}
	
		// now we need to write this into a file
		$filename = KREXX_DIR . $logDir . DIRECTORY_SEPARATOR . $timestamp . '.kreXX.html';
		// file_put_contents($filename, $buffer);
		file_put_contents($filename, $output, FILE_APPEND);
	}
	

	/**
	 * Simply outputs the Header of kreXX
	 *
	 * @param string $headline
	 * @return string
	 */
	public static function outputHeader($headline) {
		static $docType = null;
	
		// Setting tamplate info
		if (is_null(kreXXRender::$skin)) {
			kreXXRender::$skin = kreXXConfig::getConfigValue('render', 'skin');
		}
		if (! isset($docType)) {
			// send doctype only once!
			$docType = kreXXConfig::getConfigValue('render', 'doctype');
			return kreXXRender::renderHeader($docType, $headline);
		} else {
			return kreXXRender::renderHeader('', $headline);
		}
	}
	
	/**
	 * Simply renders the footer and output current settings
	 *
	 * @param string $caller Where was kreXX initially invoced from
	 * @todo Refactor, this function is by far too big
	 */
	public static function outputFooter($caller) {
	
		// Wrap an expandable arround to save space
		$anonFunction = function ($params) {
			$config = $params[0];
			$source = $params[1];
			$configOutput = '';
			foreach ($config as $sectionName => $sectionData) {
				// echo '<pre>';print_r($sectionData);die('</pre>');
				$paramsExpandable = array (
						$sectionName,
						$sectionData,
						$source[$sectionName]
				);
	
				// Render a whole section
				$anonfunction = function ($params) {
					$sectionName = $params[0];
					$sectionData = $params[1];
					$source = $params[2];
					$sectionOutput = '';
					foreach ($sectionData as $parameterName => $parameterValue) {
						// Render the single value
						// We need to find out where the value comes from
						if ($sectionName != 'deep') {
							// render normal
							$sectionOutput .= kreXXRender::renderSingleChild($parameterValue, $parameterName, htmlspecialchars($parameterValue), false, $source[$parameterName], '. . .');
						} else {
							// render editable
							switch ($parameterName) {
								case "analyseProtected":
									$type = 'Select';
									break;
								case "analysePrivate":
									$type = 'Select';
									break;
								case "analyseMethods":
									$type = 'Select';
									break;
								case "analyseTraversable":
									$type = 'Select';
									break;
								case "debugMethods":
									$type = 'Input';
									break;
								case "level":
									$type = 'Input';
									break;
								default:
									$type = 'Input';
									break;
							}
							$sectionOutput .= kreXXRender::renderSingleEditableChild($parameterName, htmlspecialchars($parameterValue), $source[$parameterName], $type, $parameterName);
						}
					}
					// Render the reset-button which will delete the debug-cookie
					if ($sectionName == 'deep') {
						$sectionOutput .= kreXXRender::renderCookieResetbutton();
					}
					return $sectionOutput;
				};
				$configOutput .= kreXXRender::renderExpandableChild($sectionName, 'Config', $anonfunction, $paramsExpandable, '. . .');
			}
			// Render the dev-handle field
			$configOutput .= kreXXRender::renderSingleEditableChild('Local open function', kreXXConfig::getDevHandler(), '\krexx::', 'Input', 'localFunction');
			return $configOutput;
		};
	
		// $path = KREXX_DIR . 'kreXX.ini';
		// $config = (array) @parse_ini_file($path, true);
	
		// now we need to stitch together the content of the ini file
		// as well as it's path
		if (!is_readable(kreXXConfig::getPathToIni())) {
			// project settings are not accessible
			// tell the user, that we are using fallback sttings
			$path = 'kreXX.ini not found, using factory settings';
			// $config = array();
		} else {
			$path = 'Current configuration';
		}
	
		$myConfig = kreXXConfig::getWholeConfiguration();
		$source = $myConfig[0];
		$config = $myConfig[1];
		
		$parameter = array (
				$config,
				$source
		);
	
	
		$configOutput = kreXXRender::renderExpandableChild($path, kreXXConfig::getPathToIni(), $anonFunction, $parameter, '', '',  'currentSettings');
		return kreXXRender::renderFooter($caller, $configOutput);
	}
	
	/**
	 * Print the skin (CSS)
	 *
	 * @return boolean
	 */
	public Static Function getCssAndJs() {
		static $_css = false;
	
		// already set ?
		//
		if ($_css) {
			return;
		}
	
		// get the css file
		//
		$_ = KREXX_DIR . 'skins/' . kreXXRender::$skin . '/skin.css';
		$css = file_get_contents($_);
		// remove whitespace
		$css =  preg_replace('/\s+/', ' ',$css);
	
	
		$js = '';
		// adding JQuery
		$jsLib = KREXX_DIR . 'jsLibs/' . kreXXConfig::getConfigValue('render', 'jsLib');
		if (file_exists($jsLib)) {
			// we are not going to minimize the 3'rd party library
			$js .= join(file($jsLib));
		}
		// kreXX.js is comes directly form the template
		$templateJs = KREXX_DIR . 'skins/' . kreXXRender::$skin . '/kreXX.js';
		if (file_exists($templateJs)) {
			$temlJs = file($templateJs);
			// remove whitespace
			$temlJs =  preg_replace('/\s+/', ' ',$temlJs);
			$js .= join(file($templateJs));
		}
	
		return kreXXRender::renderCssJs($css, $js);
	
	}
	
	/**
	 * Generates a id for the DOM
	 * This is used to jump from a Recussion to the object analysis data
	 * The ID is the object hash as well as the kruXX call number, to avoit
	 * collusions (even if they are unlikely).
	 *
	 * @param mixed $data
	 * @return string the generated id
	 */
	public static function generateDomIdFromObject($data) {
		if (is_object($data)) {
			return kreXXRender::$kreXXCount . '_' . spl_object_hash($data);
		} else {
			// do nothing
		}
	}
	
}
