<?php

/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 *
 *
 *
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */
namespace kreSpace;


/**
 * kreXX toolbox
 *
 * This class hosts the variable analysis functions
 *
 * @package kreXX
 */
class kreXXVariables {
	
	/**
	 * Render a dump for a NULL value
	 *
	 * @param string $name
	 * @return string
	 */
	public Static Function analyseNull($name) {
		$data = 'NULL';
		return kreXXRender::renderSingleChild($data, $name, $data, false, 'null');
	}
	
	/**
	 * Render a dump for an array
	 *
	 * @param mixed $data
	 * @param string $name
	 */
	public Static Function analyseArray(&$data, $name) {
		// Dumping all Properties
		$parameter = array (
				$data
		);
		$anonFunction = function ($parameter) {
			$data = $parameter[0];
			return kreXXInternals::interateThrough($data);
		};
		// }
	
		return kreXXRender::renderExpandableChild($name, 'array', $anonFunction, $parameter, count($data) . ' elements');
	}
	
	/**
	 *
	 * @param mixed $data
	 * @param string $name
	 */
	public Static Function analyseResource($data, $name) {
		$data = get_resource_type($data);
		return kreXXRender::renderSingleChild($data, $name, $data, false, 'resource');
	}
	
	/**
	 * Render a dump for a boolean value
	 *
	 * @param mixed $data
	 * @param string $name
	 */
	public Static Function analyseBoolean($data, $name) {
		$data = $data ? 'TRUE' : 'FALSE';
		return kreXXRender::renderSingleChild($data, $name, $data, false, 'boolean');
	}
	
	/**
	 * Render a dump for a integer value
	 *
	 * @param mixed $data
	 * @param string $name
	 */
	public Static Function analyseInteger($data, $name) {
		return kreXXRender::renderSingleChild($data, $name, $data, false, 'integer');
	}
	
	/**
	 * Render a dump for a float value
	 *
	 * @param mixed $data
	 * @param string $name
	 */
	public Static Function analyseFloat($data, $name) {
		return kreXXRender::renderSingleChild($data, $name, $data, false, 'float');
	}
	
	/**
	 * Render a dump for a string value
	 *
	 * @param mixed $data
	 * @param string $name
	 */
	public Static Function analyseString($data, $name) {
	
		// extra ?
		$_extra = false;
		$_ = $data;
		if (strLen($data) > KREXX_TRUNCATE_LENGTH) {
			$_ = substr($data, 0, KREXX_TRUNCATE_LENGTH - 3) . '...';
			$_extra = true;
		}
	
		// security, there could be anything inside the string
		$data = nl2br(htmlSpecialChars($data));
		$_ = nl2br(htmlSpecialChars($_));
	
		return kreXXRender::renderSingleChild($data, $name, $_, $_extra, 'string');
	}
	
	
}