/**
 * JavaScript routines for kreXX
 * 
 * kreXX is a debugging tool (PHP5.3 only), which displays structured
 * information about any PHP object. It is a nice replacement for print_r() or
 * var_dump() which are used by a lot of PHP developers.
 * 
 * @author brainworXX GmbH <info@brainworxx.de>
 * 
 * kreXX is a fork of Krumo, which was originally written by:
 * Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 * 
 * @license http://opensource.org/licenses/lgpl-license.php GNU Lesser General
 *          Public License Version 2.1
 * 
 * @package kreXX
 */ 


if (window.jQuery) {
	jQuery.noConflict();
}

/**
 * kreXX JS Class
 */
function kreXX() {
}



/**
 * Add a CSS class to an HTML element
 * 
 * @param HtmlElement el
 * @param string className
 * @return void
 */
kreXX.reclass = function(el, className) {
	if (el.className.indexOf(className) < 0) {
		el.className += (' ' + className);
	}
};


/**
 * Remove a CSS class to an HTML element
 * 
 * @param HtmlElement  el
 * @param string className
 * @return void
 */
kreXX.unclass = function(el, className) {
	if (el.className.indexOf(className) > -1) {
		el.className = el.className.replace(className, '');
	}
};



/**
 * Toggle the nodes connected to an HTML element
 * 
 * @param HtmlElement el
 * @return void
 */
kreXX.toggle = function(el) {
	var ul = el.parentNode.getElementsByTagName('ul');
	for (var i = 0; i < ul.length; i++) {
		if (ul[i].parentNode.parentNode == el.parentNode) {
			ul[i].parentNode.style.display = (ul[i].parentNode.style.display == 'none') ? 'block'
					: 'none';
		}
	}

	/*  toggle class */
	if (ul[0].parentNode.style.display == 'block') {
		kreXX.reclass(el, 'kreXX-opened');
	} else {
		kreXX.unclass(el, 'kreXX-opened');
	}
};



/**
 * Hover over an HTML element
 * Display helptext, if aviable
 * 
 * @param HtmlElement el
 * @return void
 */
kreXX.over = function(el) {

	kreXX.showHelp(el);

	kreXX.reclass(el, 'kreXX-hover');
	kreXX.unclass(el, 'kreXX-highlight');
};



/**
 * Hover out an HTML element
 * 
 * @param HtmlElement el
 * @return void
 */

kreXX.out = function(el) {
	kreXX.hideHelp();
	kreXX.unclass(el, 'kreXX-hover');
};

/**
 * Expand kreXX child and then focus on it
 * 
 * @param string dimId
 * @return void
 */
kreXX.jumpTo = function(domId) {
	if (window.jQuery) {
		(function($) {
			$(function() {
				var $nests = $('#' + domId).parents('.kreXX-nest');
				/* show them */
				$nests.show();
				/* expand parents */
				$nests.prev().addClass('kreXX-opened');
				/* mark it and scroll into view */
				var $expandableElement = $('#' + domId).prev();
				/* original element found? */
				if ($expandableElement.length) {
					$expandableElement.addClass('kreXX-highlight');
					$('html, body').animate({
						scrollTop : $expandableElement.offset().top
					}, 5);
				}
			});
		})(jQuery);
	}
};

/**
 * show the helptext as a tooltip
 * 
 * @param el
 */
kreXX.showHelp = function(el) {
	if (window.jQuery) {
		(function($) {
			$(function() {
				var $el = $(el);
				var helpText =  $el.data('help');
				if (typeof helpText != 'undefined' && helpText.length > 0) {
					$el.append('<div class="krexxhelp">'+helpText+('help')+'</div>');
				}
			});
		})(jQuery);
	}
};
kreXX.hideHelp = function() {
	if (window.jQuery) {
		(function($) {
			$(function() {
				$('.krexxhelp').remove();
			});
		})(jQuery);
	}
};

if (window.jQuery) {
	(function($) {
		$(function() {
			$.fn.draXX = function(handle) {

				var selector = this.selector,
					$handle = this.find(handle);

				return $handle.on("mousedown",function(e) {
				
					var $content = $(this).parents(selector),
						/* calculate original offset */
						offSetY = $content.offset().top + $content.outerHeight() - e.pageY - $content.outerHeight(),
						offSetX = $content.offset().left + $content.outerWidth() - e.pageX - $content.outerWidth();
					
					$(document).on("mousemove", function(e) {
						/* set new offset according to mouseposition */
						$content.offset({
							top : e.pageY + offSetY,
							left : e.pageX + offSetX
						})
						.on("mouseup",	function() {
							/* unregister to prevent slowdown */
							$(document).off("mousemove");
						});
					});
					/* do not trigger a "select" on the handle */
					e.preventDefault();
				});		
			};

			/* initialize the draggable */
			$('.kreXX-wrapper').draXX('.kreXX-headnote');
		
			/* register functions for the local dev-settings */
			$('.kreXX-editable select, .kreXX-editable input').on("change", function(e) {
				/* get old value */
				var settings = readSettings();
				
				/* get new settings from element */
				var $this = $(this);
				var newValue = $this.val();
				var valueName = $this.attr('name');
				settings[valueName] = newValue;
				
				/* save it */
				var date = new Date();
			    date.setTime(date.getTime() + (99*24*60*60*1000));
			    var expires = 'expires=' + date.toUTCString();
				document.cookie = 'kreXXDebugSettings=' + JSON.stringify(settings) + '; ' + expires;
				
				/* Feedback about update */
				alert (valueName + ' --> ' + newValue + '\n\nPlease reload the page to use the new local settings.');
			});
			
			/* register cookie reset function on the reset button */
			$('.kreXX-resetbutton').on('click', function(e){
				/* we do not delete the cookie, we simply remove all settings in it */
				/* exept the developer handle */
				var oldSettings = readSettings();
				var settings = {};
				settings['Local open function'] = oldSettings['Local open function'];
				var date = new Date();
			    date.setTime(date.getTime() + (99*24*60*60*1000));
			    var expires = 'expires=' + date.toUTCString();
				document.cookie = 'kreXXDebugSettings=' + JSON.stringify(settings) + '; ' + expires;
				
				alert('All local configuration have been reset.\n\nPlease reload the page to use the global settings.');
			});
			
			/* disable form-buttons in case a logfile is opened local */
			$( document ).ready(function() {
				if (window.location.protocol == 'file:') {
					$('.kreXX-editable').children().prop('disabled', true);
					$('.kreXX-resetbutton').prop('disabled', true);
				}
			});
			
			function readSettings() {
			    var cookieName = "kreXXDebugSettings=";
			    var cookieArray = document.cookie.split(';');
			    for (var i = 0; i < cookieArray.length; i++) {
			        var c = cookieArray[i];
			        while (c.charAt(0) === ' '){
			        	c = c.substring(1, c.length);
			        }
			        if (c.indexOf(cookieName) === 0) {
			        	return JSON.parse(c.substring(cookieName.length, c.length));
			        	
			        }
			    }
			    var myObject = {};
			    return myObject;
			}
		});
	})(jQuery);
}